@@ -104,7 +104,7 @@ from GangaCore.Utility.GridShell import getShell
 
 logger = getLogger()
 
-allowed_exit_range = range(1000)
+allowed_exit_range = list(range(1000))
 
 
 class GridCommand(ICommandSet):
@@ -392,7 +392,7 @@ class GridProxy(ICredential):
             self.chooseCommandSet()
             infoList = [self.command.info]
             # Append option value pairs
-            for optName, optVal in self.command.infoOpts.iteritems():
+            for optName, optVal in self.command.infoOpts.items():
                 infoList.append("%s %s" % (optName, optVal))
             logger.debug("Executing timeHMS Command: %s" % " ".join(infoList))
             status, output, message = self.shell.cmd1\
