@@ -7,7 +7,7 @@ import mimetypes
 import shutil
 from collections import defaultdict
 
-from urlparse import urlparse
+from urllib.parse import urlparse
 
 from GangaCore.Core.GangaThread.MTRunner import MTRunner, Data, Algorithm
 from GangaCore.Core.exceptions import GangaException
@@ -34,7 +34,7 @@ def __cream_resolveOSBList__(job, jdl):
 
     osbURIList = []
 
-    re_osb = re.compile('^.*OutputSandbox\s+\=\s+\{(.*)\}\s?\]?$')
+    re_osb = re.compile(r'^.*OutputSandbox\s+\=\s+\{(.*)\}\s?\]?$')
 
     for l in jdl.split(';'):
         m = re_osb.match(l)
@@ -357,7 +357,7 @@ class CREAM(IBackend):
             # submitted jobs on WMS immediately
             logger.error(
                 'some bulk jobs not successfully (re)submitted, canceling submitted jobs on WMS')
-            Grid.cancel_multiple(runner.getResults().values())
+            Grid.cancel_multiple(list(runner.getResults().values()))
             return None
         else:
             return runner.getResults()
@@ -441,8 +441,8 @@ def execSyscmdSubprocess(cmd, wdir=os.getcwd()):
 
     global exitcode
 
-    outfile   = file('stdout','w')
-    errorfile = file('stderr','w')
+    outfile   = open('stdout','w')
+    errorfile = open('stderr','w')
 
     try:
         child = subprocess.Popen(cmd, cwd=wdir, shell=True, stdout=outfile, stderr=errorfile)
@@ -994,7 +994,7 @@ sys.exit(0)
 
         if node_jids:
             for sj in rjobs:
-                if sj.id in node_jids.keys():
+                if sj.id in node_jids:
                     sj.backend.id = node_jids[sj.id]
                     sj.backend.CE = self.CE
                     sj.backend.actualCE = sj.backend.CE
@@ -1031,7 +1031,7 @@ sys.exit(0)
 
         if node_jids:
             for sj in rjobs:
-                if sj.id in node_jids.keys():
+                if sj.id in node_jids:
                     self.__refresh_jobinfo__(sj)
                     sj.backend.id = node_jids[sj.id]
                     sj.backend.CE = self.CE
