@@ -53,9 +53,7 @@ class GangaPrompt(Prompts):
 
     def in_prompt_tokens(self, cli = None):
         from datetime import datetime
-        from GangaCore.Utility.logging import flushAtIPythonPrompt
-        __flushCmd = flushAtIPythonPrompt()
-        return [(Token.Prompt,  str(__flushCmd)+'['+str(datetime.now().strftime("%H:%M:%S"))+']\nGanga In ['.format(GangaPrompt.environment)),
+        return [(Token.Prompt, '['+str(datetime.now().strftime("%H:%M:%S"))+']\nGanga In ['.format(GangaPrompt.environment)),
                 (Token.PromptNum, str(self.shell.execution_count)),
                 (Token.Prompt, ']: ')]
 
@@ -86,9 +84,9 @@ DEBUGFILES = False
 MONITOR_FILES = False
 
 if DEBUGFILES or MONITOR_FILES:
-    import __builtin__
+    import builtins
     openfiles = {}
-    oldfile = __builtin__.file
+    oldfile = builtins.file
 
     class newfile(oldfile):
 
@@ -111,7 +109,7 @@ if DEBUGFILES or MONITOR_FILES:
             #openfiles[ self.x ] = None
             del openfiles[self.x]
 
-    oldopen = __builtin__.open
+    oldopen = builtins.open
 
     def newopen(*args):
         if DEBUGFILES:
@@ -119,19 +117,19 @@ if DEBUGFILES or MONITOR_FILES:
             logger = getLogger()
             logger.debug("NewOpen")
         return newfile(*args)
-    __builtin__.file = newfile
-    __builtin__.open = newopen
+    builtins.file = newfile
+    builtins.open = newopen
 
     def printOpenFiles():
         from GangaCore.Utility.logging import getLogger
         logger = getLogger()
         logger.debug("### %d OPEN FILES: [%s]" % (len(openfiles), ", ".join(
-            str(f) for f in openfiles.keys() if openfiles[f] is not None)))
+            str(f) for f in openfiles if openfiles[f] is not None)))
 
     safeFiles = ['.ganga.log', '.gangarc', 'ipythonrc', 'history', 'persist']
 
     def safeCloseOpenFiles():
-        for f in openfiles.keys():
+        for f in list(openfiles.keys()):
             if f not in safeFiles:
                 openfiles[f].close()
 
@@ -156,9 +154,10 @@ def manualExportToGPI(my_interface=None):
     exportToInterface(my_interface, 'ReadOnlyObjectError', ReadOnlyObjectError, 'Exceptions')
     exportToInterface(my_interface, 'JobError', JobError, 'Exceptions')
 
-    from GangaCore.Runtime.GPIFunctions import license, typename, categoryname, plugins, convert_merger_to_postprocessor
+    from GangaCore.Runtime.GPIFunctions import license, typename, categoryname, plugins, convert_merger_to_postprocessor, runfile
 
     exportToInterface(my_interface, 'license', license, 'Functions')
+    exportToInterface(my_interface, 'runfile', runfile, 'Functions')
     # FIXME:
     #from GangaCore.Runtime.GPIFunctions import applications, backends, list_plugins
     #exportToInterface(my_interface, 'applications', applications, 'Functions)
@@ -453,7 +452,7 @@ under certain conditions; type license() for details.
                 if not os.path.exists(bn):
                     try:
                         os.rename(config_file, bn)
-                    except Exception, err:
+                    except Exception as err:
                         logger.error('Failed to create config backup file %s' % bn)
                         logger.error('Old file will not be overwritten, please manually remove it and start ganga with the -g option to re-generate it')
                         logger.error('Reason: %s' % err)
@@ -497,7 +496,7 @@ under certain conditions; type license() for details.
         import itertools
         logger = getLogger('ReleaseNotes')
         if getConfig('Configuration')['ReleaseNotes'] == True:
-            packages = itertools.imap(lambda x: 'ganga/ganga/' + x, itertools.ifilter(lambda x: x != '', ['GangaCore'] + getConfig('Configuration')['RUNTIME_PATH'].split(':')))
+            packages = map(lambda x: 'ganga/ganga/' + x, filter(lambda x: x != '', ['GangaCore'] + getConfig('Configuration')['RUNTIME_PATH'].split(':')))
             pathname = os.path.join(os.path.dirname(__file__), '..', '..', 'GangaRelease', 'ReleaseNotes-%s' % _gangaVersion)
 
             if not os.path.exists(pathname):
@@ -509,7 +508,7 @@ under certain conditions; type license() for details.
             with open(pathname, 'r') as f:
                 try:
                     notes = [l.strip() for l in f.read().replace(bounding_line, '').split(dividing_line)]
-                except Exception, err:
+                except Exception as err:
                     logger.error('Error while attempting to read release notes')
                     logger.debug('Reason: %s' % err)
                     raise
@@ -577,13 +576,13 @@ under certain conditions; type license() for details.
             # the user actually sees this message last
             time.sleep(3.)
             if interactive:
-                yes = raw_input('Would you like to create default config file ~/.gangarc with standard settings ([y]/n) ?\n')
+                yes = input('Would you like to create default config file ~/.gangarc with standard settings ([y]/n) ?\n')
             else:
                 yes = 'y'
             if yes.lower() in ['', 'y']:
                 self.generate_config_file(default_config, interactive)
                 if interactive:
-                    raw_input('Press <Enter> to continue.\n')
+                    input('Press <Enter> to continue.\n')
         elif bugVer:
             self.print_release_notes()
             self.rollHistoryForward()
@@ -655,7 +654,7 @@ under certain conditions; type license() for details.
         try:
             with open(config_file) as cf:
                 first_line = cf.readline()
-                r = re.compile('# Ganga configuration file \(\$[N]ame: (?P<version>\S+) \$\)').match(first_line)
+                r = re.compile(r'# Ganga configuration file \(\$[N]ame: (?P<version>\S+) \$\)').match(first_line)
                 this_logger = getLogger("Configure")
                 if not r:
                     this_logger.error('file %s does not seem to be a Ganga config file', config_file)
@@ -664,9 +663,6 @@ under certain conditions; type license() for details.
                     cv = r.group('version').split('-')  #Version number is in Ganga-x-y-z format
                     if len(cv) == 1:
                         cv = new_version_format_to_old(cv[0]).split('-')
-                    if cv[1] != '7':
-                        this_logger.error('file %s was created by a development release (%s)', config_file, r.group('version'))
-                        this_logger.error('try -g option to create valid ~/.gangarc')
         except IOError as x:
             # ignore all I/O errors (e.g. file does not exist), this is just an
             # advisory check
@@ -688,7 +684,6 @@ under certain conditions; type license() for details.
 
         if config_path:
             config_path = GangaCore.Utility.files.expandfilename(os.path.join(GangaRootPath, config_path))
-
         # check if the specified config options are different from the defaults
         # and set session values appropriately
         syscfg = getConfig("System")
@@ -706,14 +701,14 @@ under certain conditions; type license() for details.
 
         def _createpath(dir):
 
-            def _accept(fname, p=re.compile('.*\.ini$')):
+            def _accept(fname, p=re.compile(r'.*\.ini$')):
                 return (os.path.isfile(fname) or os.path.islink(fname)) and p.match(fname)
 
             files = []
             if dir and os.path.exists(dir) and os.path.isdir(dir):
                 files = [os.path.join(dir, f) for f in os.listdir(dir) if
                          _accept(os.path.join(dir, f))]
-            return string.join(files, os.pathsep)
+            return os.pathsep.join(files)
 
         def _versionsort(s, p=re.compile(r'^(\d+)-(\d+)-*(\d*)')):
             m = p.match(s)
@@ -724,7 +719,7 @@ under certain conditions; type license() for details.
                     return int(m.group(1)), int(m.group(2)), int(m.group(3))
             if s == 'SVN':
                 return 'SVN'
-            return None
+            return 0, 0, 0
 
         if "GANGA_SITE_CONFIG_AREA" in os.environ:
             this_dir = os.environ['GANGA_SITE_CONFIG_AREA']
@@ -864,10 +859,10 @@ under certain conditions; type license() for details.
             if not os.path.exists(os.path.join(config['gangadir'], "server")):
                 os.makedirs(os.path.join(config['gangadir'], "server"))
 
-            si = file("/dev/null", 'r')
-            so = file(os.path.join(
+            si = open("/dev/null", 'r')
+            so = open(os.path.join(
                 config['gangadir'], "server", "server-%s.stdout" % (os.uname()[1])), 'a')
-            se = file(os.path.join(
+            se = open(os.path.join(
                 config['gangadir'], "server", "server-%s.stderr" % (os.uname()[1])), 'a', 0)
 
             os.dup2(si.fileno(), sys.stdin.fileno())
@@ -896,7 +891,7 @@ under certain conditions; type license() for details.
         except Exception as x:
             logger.critical('Ganga system plugins could not be loaded due to the following reason: %s', x)
             logger.exception(x)
-            raise GangaException(x), None, sys.exc_info()[2]
+            raise GangaException(x).with_traceback(sys.exc_info()[2])
 
         initSetupRuntimePackages()
 
@@ -1034,7 +1029,7 @@ under certain conditions; type license() for details.
         fileName = fullpath('~/.ganga.py')
         if os.path.exists(fileName):
             try:
-                execfile(fileName, local_ns)
+                exec(compile(open(fileName).read(), fileName, 'exec'), local_ns)
             except Exception as x:
                 logger.error('Failed to source %s (Error was "%s"). Check your file for syntax errors.', fileName, x)
         # exec StartupGPI code
@@ -1044,9 +1039,10 @@ under certain conditions; type license() for details.
             # ConfigParser trims the lines and escape the space chars
             # so we have only one possibility to insert python code :
             # using explicitly '\n' and '\t' chars
+#FIXME: Waiting for new site config to update print to python3
             code = config['StartupGPI'].replace(
                 '\\t', '\t').replace('\\n', '\n')
-            exec code in local_ns
+            exec(code, local_ns)
 
         logger.debug("loaded .ganga.py")
 
@@ -1097,7 +1093,7 @@ under certain conditions; type license() for details.
                     setConfigOption('PollThread','forced_shutdown_policy', 'timeout')
                 from GangaCore.Core.GangaThread import GangaThreadPool
                 GangaThreadPool.shutdown_policy = 'batch'
-                execfile(script, local_ns)
+                exec(compile(open(script).read(), script, 'exec'), local_ns)
             else:
                 logger.error("'%s' not found" % self.args[0])
                 logger.info("Searched in path %s" % path)
@@ -1180,7 +1176,7 @@ under certain conditions; type license() for details.
             os.makedirs(os.environ['IPYTHONDIR'])
 
         rc_file = os.path.join(os.environ['IPYTHONDIR'], 'ipythonrc')
-	logger.debug("Checking: %s" % rc_file)
+        logger.debug("Checking: %s" % rc_file)
         if not os.path.isfile(rc_file):
             lock = open(rc_file, "w")
             lock.close()
@@ -1263,6 +1259,9 @@ under certain conditions; type license() for details.
 
     @staticmethod
     def ganga_prompt(_=None):
+        #Flush the logging
+        from GangaCore.Utility.logging import flushAtIPythonPrompt
+        flushAtIPythonPrompt()
 
         try:
             from GangaCore.GPIDev.Credentials import get_needed_credentials
