@@ -196,6 +196,23 @@ def makeConfig(name, docstring, **kwds):
     c._config_made = True
     return c
 
+def makeConfigForTest(name, docstring, **kwds):
+    """
+    Copy of makeConfig(), but without the bootstrap restrictions as Config Sections created for the Tests
+    can be added after bootstrap and should not contain any critical config.
+    """
+
+    try:
+        c = allConfigs[name]
+        c.docstring = docstring
+        for k in kwds:
+            setattr(c, k, kwds[k])
+    except KeyError:
+        c = allConfigs[name] = PackageConfig(name, docstring, **kwds)
+
+    c._config_made = True
+    return c
+
 
 class ConfigOption(object):
 
@@ -753,12 +770,12 @@ class PackageConfig(object):
                 del self.options[o]
 
 try:
-    import ConfigParser
-    GangaConfigParser = ConfigParser.SafeConfigParser
+    import configparser
+    GangaConfigParser = configparser.ConfigParser
 except ImportError:
     # For Python 3
     import configparser as ConfigParser
-    GangaConfigParser = ConfigParser.ConfigParser
+    GangaConfigParser = configparser.ConfigParser
 
 
 def make_config_parser(system_vars):
@@ -844,7 +861,7 @@ def read_ini_files(filenames, system_vars):
         logger.info('reading config file %s', f)
         try:
             with open(f) as file_f:
-                cc.readfp(file_f)
+                cc.read_file(file_f)
         except Exception as x:
             logger.warning('Exception reading config file %s', x)
 
@@ -855,7 +872,7 @@ def read_ini_files(filenames, system_vars):
             for name in cc.options(sec):
                 try:
                     value = cc.get(sec, name)
-                except (ConfigParser.InterpolationMissingOptionError, ConfigParser.InterpolationSyntaxError) as err:
+                except (configparser.InterpolationMissingOptionError, configparser.InterpolationSyntaxError) as err:
                     logger.debug("Parse Error!:\n  %s" % err)
                     value = cc.get(sec, name, raw=True)
                     #raise err
@@ -878,9 +895,9 @@ def read_ini_files(filenames, system_vars):
 
                 try:
                     current_value = main.get(sec, name)
-                except ConfigParser.NoOptionError:
+                except configparser.NoOptionError:
                     current_value = None
-                except (ConfigParser.InterpolationMissingOptionError, ConfigParser.InterpolationSyntaxError) as err:
+                except (configparser.InterpolationMissingOptionError, configparser.InterpolationSyntaxError) as err:
                     logger.debug("Parse Error!:\n  %s" % err)
                     logger.debug("Failed to expand, Importing value %s:%s as raw" % (sec, name))
                     current_value = main.get(sec, name, raw=True)
@@ -947,6 +964,32 @@ def setUserValue(config_name, option_name, value):
     # is added
     unknownUserConfigValues[config_name][option_name] = value
 
+def setUserValueForTest(config_name, option_name, value):
+    """
+    Sets the user value for the given config and option for performing Tests.
+    Used by (GangaCore/testlib/GangaUnitTest)
+    If config_name exists & it's option_name also exists then modifies the option.
+    If config_name exists & it's option_name does not exists but options are allowed to be added to config_name (.is_open=True) then adds the option.
+    If config_name exists & it's option_name does not exists but options are not allowed to be added to config_name (.is_open=False) then doesn't add the option.
+    If config_name does not exists then creates the config_name (with .is_open=True) and add option_name to it. <--- Created for testing purpose
+    """
+
+    if config_name in allConfigs:
+        c = getConfig(config_name)
+        if option_name in c.options:
+            c.setUserValue(option_name, value)
+            return
+        if c.is_open:
+            c._addOpenOption(option_name, value)
+            c.setUserValue(option_name, value)
+            return
+
+    if config_name not in allConfigs:
+        c = makeConfigForTest(config_name, '')
+        c.is_open = True
+        c._addOpenOption(option_name, value)
+        c.setUserValue(option_name, value)
+
 
 def setSessionValue(config_name, option_name, value):
     """
@@ -1017,7 +1060,7 @@ def setSessionValuesFromFiles(filenames, system_vars):
                 continue
             try:
                 v = grcCfg.get(name, o)
-            except (ConfigParser.InterpolationMissingOptionError, ConfigParser.InterpolationSyntaxError) as err:
+            except (configparser.InterpolationMissingOptionError, configparser.InterpolationSyntaxError) as err:
                 logger = getLogger()
                 logger.debug("Parse Error!:\n  %s" % err)
                 logger.warning("Can't expand the config file option %s:%s, treating it as raw" % (name, o))
@@ -1037,7 +1080,7 @@ def setSessionValuesFromFiles(filenames, system_vars):
                 continue
             try:
                 v = cfg.get(name, o)
-            except (ConfigParser.InterpolationMissingOptionError, ConfigParser.InterpolationSyntaxError) as err:
+            except (configparser.InterpolationMissingOptionError, configparser.InterpolationSyntaxError) as err:
                 logger = getLogger()
                 logger.debug("Parse Error!:\n  %s" % err)
                 logger.warning("Can't expand the config file option %s:%s, treating it as raw" % (name, o))
@@ -1069,7 +1112,7 @@ def load_user_config(filename, system_vars):
                 continue
             try:
                 v = new_cfg.get(name, o)
-            except (ConfigParser.InterpolationMissingOptionError, ConfigParser.InterpolationSyntaxError) as err:
+            except (configparser.InterpolationMissingOptionError, configparser.InterpolationSyntaxError) as err:
                 logger.debug("Parse Error!:\n  %s" % err)
                 logger.debug("Failed to expand %s:%s, loading it as raw" % (name, o))
                 v = new_cfg.get(name, o, raw=True)
@@ -1133,9 +1176,9 @@ def sanityCheck():
 
         if not cfg.is_open:
             if opts:
-                logger.error("unknown options [%s]%s", name, ','.join(opts.keys()))
+                logger.error("unknown options [%s]%s", name, ','.join(list(opts.keys())))
         else:
             # add all options for open sections
-            for o, v in zip(opts.keys(), opts.values()):
+            for o, v in zip(list(opts.keys()), list(opts.values())):
                 cfg._addOpenOption(o, v)
                 cfg.setSessionValue(o, v)
