@@ -3,7 +3,7 @@ import os
 import re
 import inspect
 import getpass
-import commands
+import subprocess
 from GangaCore.Utility.ColourText import ANSIMarkup, overview_colours
 
 # Global Variable to enable Job Sharing mechanism required in GANGA SWAN INTEGRATION.
@@ -28,7 +28,7 @@ def getLCGRootPath():
 
 # ------------------------------------------------
 # store Ganga version based on new git tag for this file
-_gangaVersion = '7.1.13'
+_gangaVersion = '8.3.0'
 _development = False
 
 # store a path to Ganga libraries
@@ -103,7 +103,8 @@ Normally you should not worry about it.
 If an element of the path is just a name (like in the example below)
 then the plugins will be loaded using current python path. This means that
 some packages such as GangaTest may be taken from the release area.""",
-        examples="RUNTIME_PATH = GangaGUIRUNTIME_PATH = /my/SpecialExtensions:GangaTest ")
+        examples="""RUNTIME_PATH = GangaGUI
+RUNTIME_PATH = /my/SpecialExtensions:GangaTest""")
 
 conf_config.addOption('TextShell', 'IPython', """ The type of the interactive shell: IPython (cooler) or Console (limited)""")
 conf_config.addOption('StartupGPI', '', 'block of GPI commands executed at startup')
@@ -127,6 +128,11 @@ conf_config.addOption('autoGenerateJobWorkspace', False, 'Autogenerate workspace
 
 conf_config.addOption('NoAfsToken', False, 'Do not require an AFS token when running on an AFS filesystem. Not recommended!')
 
+conf_config.addOption('Profile_Memory', False, 'Run memory profiler on Ganga Objects')
+conf_config.addOption('Profile_CPU', False, 'Run cpu profiler on Ganga Objects')
+conf_config.addOption('Count_Calls', False, 'Run function call counters on Ganga Objects')
+conf_config.addOption('UDockerlocation', '~', 'Directory where udocker will be installed for local jobs if used for virtualization')
+
 # add named template options
 conf_config.addOption('namedTemplates_ext', 'tpl',
                  'The default file extension for the named template system. If a package sets up their own by calling "establishNamedTemplates" from python/Ganga/GPIDev/Lib/Job/NamedJobTemplate.py in their ini file then they can override this without needing the config option')
@@ -163,6 +169,7 @@ makeConfig("Shell", "configuration parameters for internal Shell utility.")
 # Queues
 queues_config = makeConfig("Queues", "configuration section for the queues")
 queues_config.addOption('Timeout', None, 'default timeout for queue generated processes')
+queues_config.addOption('ShutDownTimeout', 0.1, 'timeout before looping again over queue to give shutdown a chance')
 queues_config.addOption('NumWorkerThreads', 5, 'default number of worker threads in the queues system')
 
 # ------------------------------------------------
@@ -223,7 +230,7 @@ poll_config.addOption('forced_shutdown_first_prompt_time', 5,
                  "User will get the FIRST prompt after N seconds, as specified by this parameter. This parameter also defines the time that Ganga will wait before shutting down, if there are only non-critical threads alive, in both interactive and batch mode.")
 
 import sys
-poll_config.addOption('HeartBeatTimeOut', sys.maxint, 'Time before the user gets the warning that a thread has locked up due to failing to update the heartbeat attribute')
+poll_config.addOption('HeartBeatTimeOut', sys.maxsize, 'Time before the user gets the warning that a thread has locked up due to failing to update the heartbeat attribute')
 
 poll_config.addOption('autoCheckCredentials', True, 'Check credentials using the monitoring loop')
 
@@ -231,6 +238,7 @@ poll_config.addOption('autoCheckCredentials', True, 'Check credentials using the
 # Feedback
 feedback_config = makeConfig('Feedback', 'Settings for the Feedback plugin. Cannot be changed during the interactive Ganga session.')
 feedback_config.addOption('uploadServer', 'http://gangamon.cern.ch/django/errorreports', 'The server to connect to')
+#feedback_config.addOption('uploadServer', 'http://ec2-52-14-218-28.us-east-2.compute.amazonaws.com/server/', 'The server to connect to')
 
 # ------------------------------------------------
 # Associations
@@ -661,6 +669,7 @@ LocalPost = {'Local': 'client',
              'Interactive': 'client',
              'LSF': 'client',
              'SGE': 'client',
+             'Slurm': 'client',
              'PBS': 'client',
              'Condor': 'client',
              'CREAM': 'client',
@@ -681,6 +690,7 @@ output_config.addOption('LocalFile',
 LCGSEBakPost = {'LSF': 'client',
                 'PBS': 'client',
                 'SGE': 'client',
+                'Slurm': 'client',
                 'Condor': 'client',
                 'LCG': 'WN',
                 'CREAM': 'WN',
@@ -704,6 +714,7 @@ diracBackPost = {'Dirac': 'submit',
                  'LSF': 'WN',
                  'PBS': 'WN',
                  'SGE': 'WN',
+                 'Slurm': 'WN',
                  'Condor': 'WN',
                  'LCG': 'WN',
                  'CREAM': 'WN',
@@ -725,6 +736,7 @@ GoogleFileBackPost = {'Dirac': 'client',
                       'LSF': 'client',
                       'PBS': 'client',
                       'SGE': 'client',
+                      'Slurm': 'client',
                       'Condor': 'client',
                       'LCG': 'client',
                       'CREAM': 'client',
@@ -783,7 +795,7 @@ protoByExperiment = {'atlas': 'root://eosatlas.cern.ch',
                      'undefined': 'root://eos.cern.ch'}
 defaultMassStorageProto = protoByExperiment[groupname]
 
-eosinstalled, prefix = commands.getstatusoutput('which eos')
+eosinstalled, prefix = subprocess.getstatusoutput('which eos')
 if not eosinstalled == 0:
     prefix = ''
     massStoragePath = ''    
@@ -797,6 +809,7 @@ massStorageBackendPost = {'LSF': 'WN',
                           'PBS': 'WN',
                           'Condor': 'WN',
                           'SGE': 'WN',
+                          'Slurm': 'WN',
                           'LCG': 'client',
                           'CREAM': 'client',
                           'ARC': 'client',
@@ -817,6 +830,7 @@ sharedFileBackendPost = {'LSF': 'WN',
                          'Dirac': 'client',
                          'PBS': 'WN',
                          'SGE': 'WN',
+                         'Slurm': 'WN',
                          'Condor': 'WN',
                          'Interactive': 'client',
                          'Local': 'WN',
@@ -825,7 +839,7 @@ sharedFileBackendPost = {'LSF': 'WN',
 output_config.addOption('SharedFile',
                        {'fileExtensions': [''],
                         'backendPostprocess': sharedFileBackendPost,
-                        'uploadOptions': {'path': None, 'cp_cmd': 'cp', 'ls_cmd': 'ls', 'mkdir_cmd': 'mkdir'},
+                        'uploadOptions': {'path': '~', 'cp_cmd': 'cp', 'ls_cmd': 'ls', 'mkdir_cmd': 'mkdir'},
                         'defaultProtocol': 'file://'},
                         docstr_Ext % ('Shared Storage', 'SharedFS'))
 
