@@ -1,3 +1,7 @@
+#Bail out when loading this module as it is not python3 compliant
+from GangaCore.Core.exceptions import PluginError
+raise PluginError("The GangaNA62 module has not been upgraded for python 3. The last python 2 Ganga version is 7.1.15 . Please contact the ganga devs to discuss updating this module.")
+
 import os
 import GangaCore.Utility.logging
 import GangaCore.Utility.Config
